using System;
using System.Data ;
using System.Collections  ;
using System.ComponentModel;

namespace SmartDS
{
	/// <summary>
	/// This SmartDS Version doesn't use DataSet metadata for Validation
	/// </summary>
	public class CSmartDS2 : DataSet 
	{
		public CSmartDS2()
		{
			this.Tables.CollectionChanged += 
				new CollectionChangeEventHandler(this.SchemaChangedHook);
		}

		

		private void SchemaChangedHook(
			object sender, System.ComponentModel.CollectionChangeEventArgs e) 
		{
			if (e.Action == CollectionChangeAction.Add ) 
			{
				if (e.Element is DataTable )  
				{
					((DataTable)e.Element).ColumnChanging +=new DataColumnChangeEventHandler(DS_ColumnChanging); 
					((DataTable)e.Element).RowChanging +=new DataRowChangeEventHandler(DS_RowChanging);
				}
			}
			else if (e.Action == CollectionChangeAction.Remove ) 
			{
				if (e.Element is DataTable ) 
				{
					((DataTable)e.Element).ColumnChanging -=new DataColumnChangeEventHandler(DS_ColumnChanging); 
					((DataTable)e.Element).RowChanging -=new DataRowChangeEventHandler(DS_RowChanging);
				}
			
			}
		}

		private void DS_ColumnChanging(object sender, DataColumnChangeEventArgs e)
		{
			OnColumnChanging(sender ,e);
		}

		private void DS_RowChanging(object sender, DataRowChangeEventArgs e)
		{
			OnRowChanging (sender ,e);
		}

		virtual protected void  OnRowChanging(object sender, DataRowChangeEventArgs e) 
		{
			if (e.Action == DataRowAction.Change ) 
			{
				if (e.Row.Table.ExtendedProperties.Contains ("ruleassemblyrow")) 
				{
					string l_erdescr;

					string l_ruleassembly = e.Row.Table.ExtendedProperties["ruleassemblyrow"].ToString ();
					string l_ruleclass= e.Row.Table.ExtendedProperties["ruleclassrow"].ToString ();
				
					Object l_obj = Activator.CreateInstance (l_ruleassembly,l_ruleclass).Unwrap(); 
				
					ValidationRulesBase.IValidateRow l_iv =  (ValidationRulesBase.IValidateRow )l_obj;
					l_iv.validateRow(e.Row,e.Action,out l_erdescr ); 
				}
			}
		}

		virtual protected void OnColumnChanging(object sender, DataColumnChangeEventArgs e) 
		{
			try 
			{
				string l_erdescr;
				if (e.Column.ExtendedProperties.Contains ("ruleassembly")) 
				{
					string l_ruleassembly = e.Column.ExtendedProperties["ruleassembly"].ToString ();
					string l_ruleclass= e.Column.ExtendedProperties["ruleclass"].ToString ();
					Object l_obj = Activator.CreateInstance (l_ruleassembly,l_ruleclass).Unwrap(); 
					ValidationRulesBase.IValidateProp l_iv =  (ValidationRulesBase.IValidateProp)l_obj;
					l_iv.validateProp(e.Row ,e.Column.ColumnName ,e.ProposedValue,out  l_erdescr ); 
				}
				else if (e.Column.ExtendedProperties.Contains ("required")) 
				{
					if (e.ProposedValue.ToString ()=="") {
						throw new Exception ("The field " + e.Column.ColumnName + " can't be empty");
					}
				}
				else if (e.Column.ExtendedProperties.Contains ("regexpconstraint")) 
				{
					int l_ret = ValidationRulesBase.CRegExValidate.ValidateRegEx (
						e.Column.ExtendedProperties["regexpconstraint"].ToString (),
						e.ProposedValue.ToString (),out l_erdescr);
					if(l_ret != 0) 
					{
						throw new Exception ("validation failed for field " + e.Column.ColumnName + ". regexp: " + e.Column.ExtendedProperties.Contains ("regexpconstraint") 
							+ " value: " + e.ProposedValue );
					}
				}
			}
			catch (Exception ex) 
			{
				throw;   
			}												
		}


		public int Validate(ArrayList p_errlist) 
		{
			int l_ret =0;
			string l_ruleassembly ;
			string l_ruleclass;
			string l_errdescr;
			Object l_obj ;
			if(ExtendedProperties.Contains ("ruleassembly")) 
			{
				l_ruleassembly = ExtendedProperties["ruleassembly"].ToString ();
				l_ruleclass= ExtendedProperties["ruleclass"].ToString ();
				l_obj = Activator.CreateInstance (l_ruleassembly,l_ruleclass).Unwrap(); 
				ValidationRulesBase.IValidateDs l_iv =  (ValidationRulesBase.IValidateDs )l_obj;
				l_ret = l_iv.validateDs(this, out l_errdescr); 
				if (l_ret!=0) 
				{
					p_errlist.Add (l_errdescr);
					return l_ret;
				}
			}
			foreach(DataTable  p_dt in this.Tables ) 
			{
				if (p_dt.ExtendedProperties.Contains ("ruleassemblytable")) 
				{
					l_ruleassembly = p_dt.ExtendedProperties["ruleassemblytable"].ToString ();
					l_ruleclass= p_dt.ExtendedProperties["ruleclasstable"].ToString ();
					l_obj = Activator.CreateInstance (l_ruleassembly,l_ruleclass).Unwrap(); 
					ValidationRulesBase.IValidateTable l_iv2=  (ValidationRulesBase.IValidateTable )l_obj;
					l_ret = l_iv2.validateTable (p_dt, out l_errdescr ); 
					if (l_ret!=0) {
						p_errlist.Add (l_errdescr);
						return l_ret;
					}
				} 
			}
			return 0;
		}
	}
}
